################################################################################
#  Copyright (C) 2006  Travis Shirk <travis@pobox.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id$
################################################################################
import os

import i18n
_ = i18n._
import ConfigParser

class ConfigException(Exception):
    '''Config exception'''

class Config(ConfigParser.RawConfigParser):

    (OPT_TYPE,
     OPT_DEFAULT,
     OPT_HELP) = range(3)

    LIST_DELIM = ';'

    def __init__(self, options):
        # Using RawConfigParser so that values containing '%' don't get
        # interpolated and throw errors when there is no value defined
        ConfigParser.RawConfigParser.__init__(self, None)

        self.__options = options

        # Initialize from schema
        for section in self.__options:
            self.add_section(section)
            options = self.__options[section]
            for opt in options:
                opt_type = options[opt][self.OPT_TYPE]
                if opt_type is unicode:
                    opt_val = options[opt][self.OPT_DEFAULT].encode('utf-8')
                else:
                    opt_val = str(options[opt][self.OPT_DEFAULT])
                self.set(section, opt, opt_val)

    def load(self, config_file, profile):
        self.profile = profile
        file_config = ConfigParser.ConfigParser()
        file_config.readfp(file(config_file, 'r'))

        # Add any config that is not in the default options
        for sect in file_config.sections():
            if not self.has_section(sect):
                self.add_section(sect)
            for opt in file_config.options(sect):
                val = file_config.get(sect, opt)
                self.set(sect, opt, val)

    def save(self, config_file):
        fp = file(config_file, 'w')
        ConfigParser.RawConfigParser.write(self, fp)
        fp.close()
        # Ensure security
        os.chmod(config_file, 0600)

    def get(self, section, option, default=''):
        try:
            val = ConfigParser.RawConfigParser.get(self, section, option)
        except ConfigParser.NoOptionError:
            val = default
        return val

    def getboolean(self, section, option, default=False):
        try:
            val = ConfigParser.RawConfigParser.getboolean(self, section, option)
        except ConfigParser.NoOptionError:
            val = default
        return val

    def getint(self, section, option, default=None):
        try:
            val = ConfigParser.RawConfigParser.getint(self, section, option)
        except ConfigParser.NoOptionError:
            val = default
        return val

    def getfloat(self, section, option, default=None):
        try:
            val = ConfigParser.RawConfigParser.getfloat(self, section, option)
        except ConfigParser.NoOptionError:
            val = default
        return val

    def getlist(self, section, option, default=[]):
        retlist = []
        try:
            val = self.get(section, option)
            for item in val.split(self.LIST_DELIM):
                item = item.strip()
                if not item:
                    continue
                retlist.append(item)
        except ConfigParser.NoOptionError:
            retlist = default
        return retlist

    def set(self, section, option, value):
        if isinstance(value, unicode):
            str_val = value.encode('utf-8')
        elif isinstance(value, list):
            str_val = ''
            for item in value:
                str_val += str(item) + self.LIST_DELIM
            # Remove trailing delimiter
            str_val = str_val[:-1]
        else:
            str_val = value

        ConfigParser.RawConfigParser.set(self, section, option, str_val)

    ### Overridden ###
    def read(self, filenames):
        raise ConfigException('Use the load method instead')
    def readfp(self, fp, filename):
        raise ConfigException('Use the load method instead')
    def write(self, fp):
        raise ConfigException('Use the save method instead')
